// Copyright: This source code is copyrighted (c) 2006 by Johan Erasmus. All rights reserved. This source code shall remain the exclusive
//            property of the author(s).
//
// Distribution: You are granted a non-exclusive, royalty-free right to produce and distribute compiled binary files that are built with any
//               of this source code unless specifically stated otherwise.
//               Without the express written consent of the author, you may not:
//               - Distribute modified versions of any source code by itself. You must include the original source code as you found it at
//                 the author(s)' web site.
//               - Sell or lease any portion of this source code. You are free to sell any of your own original code that works with or
//                 enhances this source code.
//               - Distribute this source code for profit.
//
// Disclaimer: This source code is provided 'as is' with no warranty of any kind.
//             The author disclaims all warranties, expressed or implied, including, without limitation, the warranties of merchantability
//             and of fitness for any purpose. Johan Erasmus assumes no liability for damages, direct or consequential, which may result
//             from the use of this source code. You also agree to not hold Johan Erasmus liable for any damages, direct or consequential
//             which may result from the use of this source code. By using this source code, you are agreeing to all of these conditions.
//
// Support: You are free to contact me, but I can not gaurantee any support whatsoever. I will however try to answer all question that are
//          directed to me.
//
// Contact Information: jerasmus@flashmail.com
//
//
// Date convention: dd/mm/yyyy
// Date Started: 21/05/2002
// Date Completed: 17/10/2006
// Date Modified: Version 1.99
//                ------------
//                04/09/2002 - Moved GetPaletteSize from protected to public
//                05/09/2002 - Added a routine to paint an icon using the two bitmaps and replacing the transparent and inverse colors with colors passed as parameters
//                06/09/2002 - Modified the constructor to use constants in the include file for initial icon size
//                07/09/2002 - Exposed FOwner to be a public Property
//                27/09/2002 - HICON is recreated each time that the bitmaps change
//                13/11/2002 - Bug Fix: 1 BPP icons did not load: Raised exception
//                             Bug Fix: BPP was not returned by the IconInfo Property: Will now allways return BPP even for icons with BPP < 8
//                02/01/2003 - Added: Support for cursors
//                03/01/2003 - Modified: Optimized a few methods for better performane
//                           - Bug Fix: Palette changes did not reflect on image
//
//                Version 2.00
//                ------------
//                03/01/2003 - Modified: Removed the dynamic array containing all the colors: Now use only windows HPALETTE and API calls for palette
//                22/03/2004 - Added: - IsIcon; IsCursor property to TIconCursorList
//                                    - GetPaletteIndexByColor to TIconCursorImage
//                                    - GetPixelCountOfColor to TIconCursorImage
//                           - Bug Fix: Icon/Cursor property was never set
//                           - Modified: Moved GetPaletteSize back to protected and created a new public property: PaletteColorCount
//                23/03/2004 - Changed: TIconCursorList's methods: IndexOf & Remove to take TIconCursorImage parameter instead of pointer
//                           - Added: Sort method for TIconCursorList
//                           - Bug Fix: LoadFromStream caused memory leak: Called TList.Clear instead of TIconCursorList.Clear
//                           - Bug Fix: LoadFromFile caused memory leak when loading invalid icon/cursor
//                18/09/2006 - Bug Fix: GetPaletteSize moved to Protected
//                13/10/2006 - Added support for icons/cursors with Alpha Transparent regions
//                17/10/2006 - Bug Fix: Paint method did not paint Alpha Transparent icons correctly due to brush handle being Canvas.Brush.Handle and not zero
//
//                Version 3.00
//                ------------
//                17/10/2006 - Released: Support for icons/cursors with Alpha Transparent regions added
//
//
//
//
// Total Devlopment Time: > 80 Hours
//
// Version: 3.00
// Compiler: Borland Delphi 5 Profesional (UP 1)
// Delphi Support: 4, 5, 6 (Tested only on 5)
// OS Platforms Supported: Win 9x, Win ME, Win NT 4, Win 2k, Win XP (ps32Bit only on XP)
// OS Platforms Tested: Win XP (SP 2)
//
//
// Project: N/A
// Description: A component to load and save Windows icon and cursor files
// Developed by: Johan Erasmus
// Credits To: Microsoft MSDN: Icons in Win32
//                             DIBs and Their Uses - Q81498
//             Delphi FAQ & TIs: Article #15890
//
// Developed for: Myself and the freeware programmer community
// Developed by: Johan Erasmus
//
// Requires: IconCursorTypes.Pas, Icon.INC
// Notes: See EOF
//
// To Compile: N/A
// Complier Errors: 0
// Compiler Warnings: 19
// Compiler Hints: 0
//
// Known bugs: None
// Known Limitations: Please see the notes at the EOF
// Future Changes: - Assign and AssignTo of TIconCursorImage is not fully functional: Has limited compatibility with TIcon as present
//


Unit Icon;

Interface

Uses Windows, Classes, SysUtils, Graphics, IconCursorTypes;


Type
  //An exception class
  EIconException = Class(Exception);


{$INCLUDE Icon.INC}
                                                           //The include file must be located at this point

Type
  TIconCursorList = Class;                                 //Forward


  //A class that represents a single icon
  TIconCursorImage = Class(TPersistent)
  Private
    FOwner             : TIconCursorList;                  //The owner of this icon: An icon list
    FHoldUpdates       : Boolean;                          //If true a new icon will not be created and it is up to the used to call UpdateIcon method after clearing this flag: It will be set to FALSE (by compiler) on creation of a new icon
    FIconCursorInfo    : TIconCursorInfo;                  //Information about the icon/cursor
    FIcon              : TIcon;                            //The actual delphi icon: This sometimes does not work properly when BPP is higer than 4
    FIconHandle        : HICON;                            //The Windows handle to the icon
    FDestroyIconHandle : Boolean;                          //A flag indicating if the destructor must free the icon handle: See CreateIconIndirect

    FIconBmp : TBitmap;                                    //The color bitmap
    FMaskBmp : TBitmap;                                    //The mask bitmap: Monochrome

    //Palette
    FPaletteSize   : TPaletteSize;                         //Palette size: One of a few pre-defined values


    //Get & Set Methods
    Procedure SetPaletteSize(Const Value : TPaletteSize);  //Set the palette size
    Function  GetPaletteColor(Index: Integer): TColor;     //Return a specific color in the palette
    Procedure SetPaletteColor(Index: Integer; Const Value: TColor);
    Function  GetIcon : TIcon;
    Function  GetIconCursorInfo : TIconCursorInfo;
    Function  GetIsIcon : Boolean;
    Function  GetIsCursor : Boolean;
    Function  GetXHotSpot : Integer;
    Procedure SetXHotSpot(Value : Integer);
    Function  GetYHotSpot : Integer;
    Procedure SetYHotSpot(Value : Integer);
    Function  GetXYHotSpot : TPoint;
    Procedure SetXYHotSpot(Value : TPoint);
    function  GetTransparentPixel(X, Y: Integer): Byte;
    procedure SetTransparentPixel(X, Y: Integer; const Value: Byte);

  Protected
    Function  CheckConstraints(Const BPP : TPaletteSize; Const Width, Height : Integer) : Boolean; Overload; Virtual;//Returns true of all parameters are valid
    Function  CheckConstraints(Const BPP, Width, Height : Integer) : Boolean; Overload; Virtual;//Returns true of all parameters are valid

    Procedure CreateMaskBitmap(Const Bitmap : TBitmap; Const Width, Height : Integer); Virtual;//Creates a mask bitmap for the icon based on the passed parameters: Bitmap must be created before calling this method
    Procedure CreateColorBitmap(Const Bitmap : TBitmap; Const Width, Height : Integer); Virtual;//Creates a icon bitmap for the icon based on the passed parameters: Bitmap must be created before calling this method

    Function  GetIconSize : Cardinal; Virtual;             //Calculate the icon's size
    Function  GetIconOffset : Cardinal; Virtual;           //Calculate the image offset of this icon in the icon list

    Procedure BitmapsChange(Sender : TObject); Virtual;    //Event handler to recreate the HICON when bitmaps change

    Function  GetPaletteSize : Integer; Virtual;           //Returns the # of colors in the palette: 0 for true color palettes

    Procedure ClearIcon(BPP : TPaletteSize; Width : Integer; Height : Integer); Virtual;//Clear an exising icon creating transparent icon

  Public
    //Constructor & Destructor
    Constructor Create(aOwner : TIconCursorList; BPP : TPaletteSize = ps4Bit; Width : Integer = cDefaultIconWidth; Height : Integer = cDefaultIconHeight); Virtual;//When you manually want to create an icon, use this constructor
    Constructor CreateFromStream(aOwner : TIconCursorList; Stream : TMemoryStream); Virtual;//When the icon is create from a stream this constructor is used and the stream must be at the start of the icon
    Destructor  Destroy; Override;

    //Methods
    Function  GetPaletteIndexByColor(Const Color : TColor) : Integer; Virtual;//Returns the palette index of the color. -1 if not found or true color
    Function  GetPixelCountOfColor(Const Color : TColor) : Integer; Virtual;//Returns the # of pixels that it coloured Color

    Procedure UpdateIcon; Virtual;                         //Updates the icon recreating the HICON GDI object from the two bitmaps: The HoldUpdates flag must be clear for this method to work
    Procedure Paint(Const aCanvas : TCanvas; Const Left, Top : Integer); Virtual;//Draw the icon to a canvas: The Canvas.Brush.Color is used as the transparent color
    Procedure SaveHeaderToStream(Const Stream : TMemoryStream); Virtual;//Save the icon header to a stream
    Procedure SaveImagesToStream(Const Stream : TMemoryStream); Virtual;//Save the icon immages to a stream
    Procedure SaveToFile(Const FileName : String); Virtual;//Save an icon to a file: This method can only be used to save a single icon to a file
    Procedure Assign(Source : TPersistent); Override;      //Allow TIcon to copy it's properties to TIconCursorImage: Can only copy a TIconCursorImage to TIconCursorImage
    Procedure AssignTo(Dest : TPersistent); Override;      //Allow TIconCursorImage to copy to TIcon: Not implimented


    //Properties
    Property IconInfo: TIconCursorInfo read GetIconCursorInfo;//Information about the icon in a structure
    Property Handle: HICON read FIconHandle;               //The Windows icon handle: Same for cursors
    Property Icon: TIcon read GetIcon;                     //A TIcon object for Delphi compatibility: It messes up high color icons


    Property IconBmp: TBitmap read FIconBmp;               //The icon bitmap
    Property MaskBmp: TBitmap read FMaskBmp;               //The mask bitmap

    Property PaletteColorCount: Integer read GetPaletteSize;//# of colors in the palette: 0 for true color palette
    Property PaletteSize: TPaletteSize read FPaletteSize write SetPaletteSize default cPaletteSize;//The size of the palette
    Property PaletteColor[Index : Integer]: TColor read GetPaletteColor write SetPaletteColor;//The colors in the palette

    Property Owner: TIconCursorList read FOwner;           //The owner of this icon
    Property HoldUpdates: Boolean read FHoldUpdates write FHoldUpdates;//If true no updates will be made

    Property IsIcon: Boolean read GetIsIcon;
    Property IsCursor: Boolean read GetIsCursor;

    Property XHotSpot: Integer read GetXHotSpot write SetXHotSpot;
    Property YHotSpot: Integer read GetYHotSpot write SetYHotSpot;
    Property XYHotSpot: TPoint read GetXYHotSpot write SetXYHotSpot;

    Property TransparentPixels[X, Y: Integer]: Byte read GetTransparentPixel write SetTransparentPixel;

  End;





  //A list of icons: TIconCursorImage
  TIconCursorList = Class(TPersistent)  
  Private
    FList         : TList;                                 //A list containing all the icon images
    FIconStream   : TMemoryStream;                         //A stream used to load and extract icons in a single icon file
    FIconOrCursor : TIconOrCursor;                         //Icon or Cursor flag
    fSortOrder    : TIconCursorSortOrder;                  //Flag indicating sort order: Ascening or descending
    fSortCriteria : TIconCursorSortCriteria;               //Sort criteria
    fOnChange     : TNotifyEvent;                          //Notify event when list changes

    //Get & Set Methods
    Function  GetIconImage(Index: Integer): TIconCursorImage;//Returns an object as TIconCursorImage from the list
    Procedure SetIconImage(Index: Integer; Const Value: TIconCursorImage);//Set an object of type TIconCursorImage
    Function  GetCount: Integer;                           //Returns the # of itmes in the list

    Function  GetIconCursor : TIconOrCursor;
    Procedure SetIconCursor(Value : TIconOrCursor);
    function  GetIsCursor: Boolean;
    function  GetIsIcon: Boolean;
    procedure SetIsCursor(const Value: Boolean);
    procedure SetIsIcon(const Value: Boolean);

  Protected
    Procedure ReadHeader; Virtual;                         //Read the icon file header
    Procedure DoChange; Virtual;                           //Do the change event 

  Public
    //Constructor & Destructor
    Constructor Create;
    Destructor  Destroy; Override;

    //Assignment
    Procedure Assign(Source : TPersistent); Override;      //Assign one list to another

    //List Methods
    Function  Add(Const BPP : TPaletteSize = ps4Bit; Const Width : Integer = 32; Const Height : Integer = 32) : TIconCursorImage; Virtual;
    Procedure Clear; Virtual;
    Procedure Delete(Const Index : Integer); Virtual;
    Function  IndexOf(Const IconImage : TIconCursorImage) : Integer; Virtual;
    Procedure Insert(Const Index : Integer; Const Icon : TIconCursorImage); Virtual;
    Function  Remove(Const IconImage : TIconCursorImage) : Integer; Virtual;
    Procedure Sort(Const SortBy : TIconCursorSortCriteria; Const SortOrder : TIconCursorSortOrder = icsoAscending); Virtual;

    //File Methods
    Procedure LoadFromFile(Const FileName : String); Virtual;//Load icon(s) from an icon file
    Procedure SaveToFile(Const FileName : String); Virtual;//Save icon(s) to an icon file
    Procedure LoadFromStream(Stream : TStream); Virtual;   //Load icon(s) from a stream
    Procedure SaveToStream(Stream : TStream); Virtual;     //Save icon(s) to a stream

    //Properties
    Property Items[Index : Integer]: TIconCursorImage read GetIconImage write SetIconImage;
    Property Count: Integer read GetCount;

    Property IsIcon: Boolean read GetIsIcon write SetIsIcon;
    Property IsCursor: Boolean read GetIsCursor write SetIsCursor;
    Property IconOrCursor: TIconOrCursor read GetIconCursor write SetIconCursor;

    Property SortCriteria: TIconCursorSortCriteria read fSortCriteria;
    Property SortOrder: TIconCursorSortOrder read fSortOrder;

    //Events
    Property OnChange: TNotifyEvent read fOnChange write fOnChange;
  End;

  

  //Paints the icon replacing the transparent parts with TransparentColor and the inverse parts with InverseColor. The result is returned in ResultBmp.
  //All 3 bitmaps must be the same size. ResultBmp must also be created prior to calling the method.
  //This routine does not really check for exceptions, since it is very unlikely: If it occurs there is no way of telling...
  Procedure PaintIcon(Const TransparentColor, InverseColor : TColor; IconBmp, MaskBmp, ResultBmp : TBitmap);




Implementation


Uses Math, Dialogs;


{$R ColorPalette.RES}


{ TIconList }  

constructor TIconCursorList.Create;
Begin
  Inherited Create;
  FList := TList.Create;                                   //Create the list
  FIconStream := TMemoryStream.Create;                     //Create the stream
  FIconOrCursor := iocIcon;                                //Allways an icon
  fSortOrder := icsoAscending;                             //Sort ascending
  fSortCriteria := icscNone;                               //No sort criteria: Keep load order
  fOnChange := Nil;                                        //Init it to no nil event handler
End;



destructor TIconCursorList.Destroy;
Begin
  FIconStream.Free;                                        //Free the stream
  Clear;                                                   //Clear the list: Freeing all icons
  FList.Free;                                              //Free the list
  Inherited Destroy;
End;



Function  TIconCursorList.GetIconCursor : TIconOrCursor;
Begin
  Result := FIconOrCursor;
End;



Procedure TIconCursorList.SetIconCursor(Value : TIconOrCursor);
Var
  Counter : Integer;                                       //Loop counter


Begin
  If Value <> FIconOrCursor Then                           //Changed?
  Begin
    FIconOrCursor := Value;                                //Update it
    If Value = iocIcon Then                                //Changed from Cursor to Icon? (Change other way around will not matter.)
    Begin
      For Counter := 0 to FList.Count - 1 do               //Loop through all items
        GetIconImage(Counter).SetXYHotSpot(Point(0, 0));   //Set hot spot to zero for icons
    End;
  End;
End;



function TIconCursorList.GetIsCursor: Boolean;
begin
  Result := FIconOrCursor = iocCursor;                     //Is it a cursor?
end;



function TIconCursorList.GetIsIcon: Boolean;
begin
  Result := FIconOrCursor = iocIcon;                       //Is it an icon?
end;



procedure TIconCursorList.SetIsCursor(const Value: Boolean);
begin
  If Value Then                                            //Cursor?
    SetIconCursor(iocCursor)                               //YES: Set it
  Else
    SetIconCursor(iocIcon);                                //NO: Set it
end;



procedure TIconCursorList.SetIsIcon(const Value: Boolean);
begin
  If Value Then                                            //Icon?
    SetIconCursor(iocIcon)                                 //YES: Set it
  Else
    SetIconCursor(iocCursor);                              //NO: Set it
end;



Procedure TIconCursorList.Assign(Source : TPersistent);
Var
  Counter : Integer;                                       //Loop counter
  Icon    : TIconCursorImage;                              //Reference to new icon image


Begin
  If Source is TIconCursorList Then                        //Only copy from one icon list to another
  Begin
    Clear;                                                 //Remove all icon images and free them
    FList.Capacity := TIconCursorList(Source).FList.Count; //Set the size of the list
    //Loop through all the items in the source and copy them
    For Counter := 0 to TIconCursorList(Source).FList.Count - 1 do
    Begin
      Icon := Add;                                         //Create a new icon
      Icon.Assign(TIconCursorList(Source).GetIconImage(Counter));//Copy the icon
    End;
    FIconOrCursor := TIconCursorList(Source).FIconOrCursor;//Set the flag
    fSortCriteria := TIconCursorList(Source).fSortCriteria;//Set the sort criteria
    fSortOrder := TIconCursorList(Source).fSortOrder;      //Set the sort order
  End
  Else
    Inherited Assign(Source);                              //Raise exception
  DoChange;                                                //Trigger change event
End;



Function TIconCursorList.Add(Const BPP : TPaletteSize = ps4Bit; Const Width : Integer = 32; Const Height : Integer = 32) : TIconCursorImage;
Begin
  Result := TIconCursorImage.Create(Self, BPP, Width, Height);   //Create a new instance of the TIconCursorImage object
  FList.Add(Result);                                       //Add it to the list
  Result.FIconCursorInfo.ImageOffset := Result.GetIconOffset;//During the execution of the constructor the icon/cursor could not calculate the image offset since it was not yet added to the list
  fSortCriteria := icscNone;                               //After item added, list must be sorted again
  DoChange;                                                //Trigger change event
End;



Procedure TIconCursorList.Delete(Const Index : Integer);
Var
  Counter : Integer;                                       //Loop Counter
  aIcon   : TIconCursorImage;                              //Reference to the TIconCursorImage object


Begin
  If NOT ((Index >= 0) AND (Index <= FList.Count - 1)) Then//Raise an exception if the index is invalid
    Raise EIconException.CreateFmt(ceListIndexError, [Index]);

  TIconCursorImage(FList.Items[Index]).Free;               //Free the icon
  FList.Delete(Index);                                     //Delete the item from the list
  //Loop through all the icons in the list updating the offsets
  For Counter := 0 to FList.Count - 1 do
  Begin
    aIcon := GetIconImage(Counter);                        //Get a reference to the IconImage object
    aIcon.FIconCursorInfo.ImageOffset := aIcon.GetIconOffset; //Update the offset, since it changed; Except if the last item was removed
  End;
  DoChange;                                                //Trigger change event
End;



Procedure TIconCursorList.Clear;
Begin
  While FList.Count <> 0 do                                //Loop through all the icons freeing one by one
  Begin
    TIconCursorImage(FList.Items[0]).Free;                 //Free the icon
    FList.Delete(0);                                       //Remove it from the list
  End;
  fSortCriteria := icscNone;                               //Clear this since it does not for empty list
  DoChange;                                                //Trigger change event
End;



Function TIconCursorList.GetCount : Integer;
Begin
  Result := FList.Count;                                   //Return the # of items in the list
End;



Function TIconCursorList.GetIconImage(Index : Integer) : TIconCursorImage;
Begin
  If NOT ((Index >= 0) AND (Index <= FList.Count - 1)) Then//Raise an exception if the index is invalid
    Raise EIconException.CreateFmt(ceListIndexError, [Index]);

  Result := TIconCursorImage(FList.Items[Index]);          //Returns an icon image object
End;



Procedure TIconCursorList.SetIconImage(Index : Integer; Const Value : TIconCursorImage);
Var
  Counter : Integer;                                       //Loop Counter
  aIcon   : TIconCursorImage;                              //Reference to the TIconCursorImage object


Begin
  If NOT ((Index >= 0) AND (Index <= FList.Count - 1)) Then//Raise an exception if the index is invalid
    Raise EIconException.CreateFmt(ceListIndexError, [Index]);

  If NOT Assigned(Value) Then
    Raise EIconException.CreateFmt(ceNoNullEntry, [Index]);


  FList.Items[Index] := Value;                             //Set the item in the list to another object: Could be dangerous...
  //Loop through all the icons in the list updating the offsets
  For Counter := 0 to FList.Count - 1 do
  Begin
    aIcon := GetIconImage(Counter);                        //Get a reference to the IconImage object
    aIcon.FIconCursorInfo.ImageOffset := aIcon.GetIconOffset;//Update the offset, since it changed; Except if the last item was removed
  End;
  fSortCriteria := icscNone;                               //After item inserted, list must be sorted again
End;



Function TIconCursorList.IndexOf(Const IconImage : TIconCursorImage) : Integer;
Begin
  Result := FList.IndexOf(IconImage);                      //Return a pointer to that object
End;



Procedure TIconCursorList.Insert(Const Index : Integer; Const Icon : TIconCursorImage);
Var
  Counter : Integer;                                       //Loop Counter
  aIcon   : TIconCursorImage;                              //Reference to the TIconCursorImage object


Begin
  If NOT ((Index >= 0) AND (Index <= FList.Count - 1)) Then//Raise an exception if the index is invalid
    Raise EIconException.CreateFmt(ceListIndexError, [Index]);

  If NOT Assigned(Icon) Then
    Raise EIconException.CreateFmt(ceNoNullEntry, [Index]);

  FList.Insert(Index, Icon);                               //Insert an icon image in the list
  //Loop through all the icons in the list updating the offsets
  For Counter := 0 to FList.Count - 1 do
  Begin
    aIcon := GetIconImage(Counter);                        //Get a reference to the IconImage object
    aIcon.FIconCursorInfo.ImageOffset := aIcon.GetIconOffset;//Update the offset, since it changed; Except if the last item was removed
  End;
  fSortCriteria := icscNone;                               //After item inserted, list must be sorted again
  DoChange;                                                //Trigger change event
End;



Function TIconCursorList.Remove(Const IconImage : TIconCursorImage) : Integer;
Var
  Counter : Integer;                                       //Loop Counter
  aIcon   : TIconCursorImage;                              //Reference to the TIconCursorImage object


Begin
  Result := FList.IndexOf(IconImage);                      //Get the index of the icon in the image list
  If NOT ((Result >= 0) AND (Result <= FList.Count - 1)) Then//Raise an exception if the index is invalid
    Raise EIconException.CreateFmt(ceListIndexError, [Result]);

  TIconCursorImage(FList.Items[Result]).Free;              //Free the icon
  FList.Remove(IconImage);                                 //Remove it from the list
  //Loop through all the icons in the list updating the offsets
  For Counter := 0 to FList.Count - 1 do
  Begin
    aIcon := GetIconImage(Counter);                        //Get a reference to the IconImage object
    aIcon.FIconCursorInfo.ImageOffset := aIcon.GetIconOffset;//Update the offset, since it changed; Except if the last item was removed
  End;
  DoChange;                                                //Trigger change event
End;



//List compare function: Handles all compares
//Since the amount of data to be sorted is small, this function does not need to be optimised by splitting it into separate compare functions
Function  IconCursorListCompare(Item1, Item2 : Pointer) : Integer;
Var
  SortCriteria : TIconCursorSortCriteria;                  //Sort criteria of list
  SortOrder    : TIconCursorSortOrder;                     //Sort order of list
  IconCursor1  : TIconCursorImage;                         //Item1 typecasted to a TIconCursorImage
  IconCursor2  : TIconCursorImage;                         //Item2 typecasted to a TIconCursorImage

  
Begin
  //Get the list's sort criteria and order:
  SortCriteria := TIconCursorImage(Item1).Owner.fSortCriteria;
  SortOrder := TIconCursorImage(Item1).Owner.fSortOrder;

  //Typecast the pointers to TIconCursorImages:
  IconCursor1 := TIconCursorImage(Item1);
  IconCursor2 := TIconCursorImage(Item2);

  //Sort criteria:
  Case SortCriteria of
    icscNone       : Result := 0;
    icscColorDepth : Begin
                       //Compare the color depth:
                       If IconCursor1.PaletteSize > IconCursor2.PaletteSize Then
                         Result := 1
                       Else
                         If IconCursor1.PaletteSize < IconCursor2.PaletteSize Then
                           Result := -1
                         Else
                           Begin
                             //Item's color depth equal: Compare the size:
                             If (IconCursor1.FIconCursorInfo.Width + IconCursor1.FIconCursorInfo.Height) > (IconCursor2.FIconCursorInfo.Width + IconCursor2.FIconCursorInfo.Height) Then
                               Result := 1
                             Else
                               If (IconCursor1.FIconCursorInfo.Width + IconCursor1.FIconCursorInfo.Height) < (IconCursor2.FIconCursorInfo.Width + IconCursor2.FIconCursorInfo.Height) Then
                                 Result := -1
                               Else
                                 Result := 0;
                           End;
                     End;
    icscSize       : Begin
                       //Compare the size by adding the widht and height:
                       If (IconCursor1.FIconCursorInfo.Width + IconCursor1.FIconCursorInfo.Height) > (IconCursor2.FIconCursorInfo.Width + IconCursor2.FIconCursorInfo.Height) Then
                         Result := 1
                       Else
                         If (IconCursor1.FIconCursorInfo.Width + IconCursor1.FIconCursorInfo.Height) < (IconCursor2.FIconCursorInfo.Width + IconCursor2.FIconCursorInfo.Height) Then
                           Result := -1
                         Else
                           Begin
                             //Item's size are equal: Compare the color depth:
                             If IconCursor1.PaletteSize > IconCursor2.PaletteSize Then
                               Result := 1
                             Else
                               If IconCursor1.PaletteSize < IconCursor2.PaletteSize Then
                                 Result := -1
                               Else
                                 Result := 0;
                           End;
                     End;
  End;

  //Check the sort order:
  If SortOrder = icsoDescending Then                       //Descending?
    Result := -Result;                                     //YES: Invert result
End;



//Sort method to sort the list by SortBy and SortOrder
procedure TIconCursorList.Sort(const SortBy: TIconCursorSortCriteria; Const SortOrder : TIconCursorSortOrder = icsoAscending);
begin
  //Set the sort criteria and order:
  fSortOrder := SortOrder;
  fSortCriteria := SortBy;

  //Sort the list based on above parameters:
  FList.Sort(IconCursorListCompare);
  DoChange;                                                //Trigger change event
end;



//Load icon(s) from a stream
procedure TIconCursorList.LoadFromStream(Stream: TStream);
begin
  FIconStream.Clear;                                       //Clear the stream
  Clear;                                                   //Clear the list, freeing all icons
  FIconStream.LoadFromStream(Stream);                      //Copy the stream
  ReadHeader;                                              //Read the header and load all the icons
  fSortCriteria := icscNone;                               //After load list must be sorted again
  DoChange;                                                //Trigger change event after load
end;



//Save icon(s) to a stream
procedure TIconCursorList.SaveToStream(Stream: TStream);
Var
  IconFileHeader : TIconCursorHeader;                      //The icon file header
  Counter        : Integer;                                //Loop counter


Begin
  //If there is no icons in the list: No file can be created
  If FList.Count < 1 Then
    Raise EIconException.CreateFmt(ceEmptyFile, []);

  FIconStream.Clear;                                       //Clear the stream

  //Fill out the icon header
  IconFileHeader.Reserved := 0;                            //Must be zero
  If FIconOrCursor = iocIcon Then
    IconFileHeader.ResType := cIcon;                       //It is an icon
  If FIconOrCursor = iocCursor Then
    IconFileHeader.ResType := cCursor;                     //It is a cursor
  IconFileHeader.Count := FList.Count;                     //The # of icons in the file

  //Save the icon file header: 6 Bytes
  FIconStream.Write(IconFileHeader, SizeOf(IconFileHeader));

  //Save all the headers to the stream
  For Counter := 0 to FList.Count - 1 do
    GetIconImage(Counter).SaveHeaderToStream(FIconStream);

  //Save all the bitmap bits to the stream
  For Counter := 0 to FList.Count - 1 do
    GetIconImage(Counter).SaveImagesToStream(FIconStream);

  //Write the icon stream to Stream passed as a parameter
  FIconStream.SaveToStream(Stream);
End;



//Load icon(s) from an icon in to the icon list
Procedure TIconCursorList.LoadFromFile(Const FileName: String);
Var
  Stream : TFileStream;                                    //A file stream to load the memory stream from


Begin
  Try
    Stream := TFileStream.Create(FileName, fmOpenRead OR fmShareDenyWrite);//Create the file stream: No other app can write to it
    LoadFromStream(Stream);                                //Do the actual loading
  Finally
    Stream.Free;                                           //Allways free it
  End;
End;



//Save the icon list to a file
Procedure TIconCursorList.SaveToFile(Const FileName : String);
Var
  Stream : TFileStream;                                    //A file stream to save the memory stream to


Begin
  Try
    Stream := TFileStream.Create(FileName, fmCreate OR fmShareExclusive);//Create the file stream: No other app is allowed to access it
    SaveToStream(Stream);                                  //Do the actual saving
  Finally
    Stream.Free;                                           //Allways free it
  End;
End;



Procedure TIconCursorList.ReadHeader;
Var
  IconHeader : TIconCursorHeader;                          //The header structure
  Icon       : TIconCursorImage;                           //The newly created icon/cursor
  Counter    : Integer;                                    //Loop counter
  ErrorMsg   : String;                                     //A list of all errors that occured when the icon was loaded


Begin
  FIconStream.Position := 0;                               //Set the position of the stream to the Beginning
  FIconStream.Read(IconHeader, SizeOf(IconHeader));        //Read the header

  //Are the bits correct: Valid Icon file?
  If (IconHeader.Reserved = cHeaderReserved) AND                          //Check that reserved byte is OK
     ((IconHeader.ResType = cCursor) OR (IconHeader.ResType = cIcon)) AND //Check the resource type: Either icon or cursor
     (IconHeader.Count > 0) AND (IconHeader.Count <= cMaxIconsInFile) Then//Check the limits: > 0
  Begin
    ErrorMsg := '';                                        //En empty string

    //Set the icon/cursor flag:
    If IconHeader.ResType = cCursor Then                   //Is it a cursor?
      FIconOrCursor := iocCursor                           //YES: Set it
    Else
      FIconOrCursor := iocIcon;                            //NO: Then is must be an icon

    For Counter := 0 to IconHeader.Count - 1 do
    Begin
      //Create a new icon: The constructor will automatically load the icon and set the stream position to the header of the next icon
      Try
        Icon := TIconCursorImage.CreateFromStream(Self, FIconStream);//Create an icon/cursor from the stream
        FList.Add(Icon);                                   //Add the icon to the list
        Icon.FIconCursorInfo.ImageOffset := Icon.GetIconOffset;//During the execution of the constructor the icon could not calculate the image offset since it was not yet added to the list
      Except
       //When an exception is raised by the constructor the error messages is saved
        On E:Exception do
          ErrorMsg := ErrorMsg + E.Message + #13 + #10;
      End;
    End;

    //Inform the user of any exceptions that occured during the load process
    If Length(ErrorMsg) <> 0 Then
      Raise EIconException.CreateFmt(ceMainLoadError + #13 + #10 + ErrorMsg, []);
  End
  Else
    Raise EIconException.CreateFmt(ceInvalidICOHeader, []);
End;



//Triggers the change event
procedure TIconCursorList.DoChange;
begin
  If Assigned(fOnChange) Then                              //Event handler assigned?
    fOnChange(Self);                                       //YES: Call it
end;







{ TIconCursorImage }

Constructor TIconCursorImage.Create(aOwner : TIconCursorList; BPP : TPaletteSize = ps4Bit; Width : Integer = 32; Height : Integer = 32);
Begin
  Inherited Create;                                        

  //Correct owner ?
  If aOwner.ClassName = 'TIconCursorList' Then             //Can not test it with If aOwner is TIconList: This will allways return true
    FOwner := aOwner                                       //Set the owner
  Else
    Raise EIconException.CreateFmt(ceInvalidOwner, [aOwner.ClassName]);

  FDestroyIconHandle := FALSE;                             //Set flag to false: Also see CreateIconIndirect & Destructor

  ClearIcon(BPP, Width, Height);                           //Create a new transparent icon
(*
  //Check if all parameters are correct
  If NOT CheckConstraints(BPP, Width, Height) Then
    Raise EIconException.CreateFmt(ceInvalidSize, [Width, Height]);

  FPaletteSize := BPP;                                     //Set palette size before creating the icon bitmap
  
  //Create the monochrome mask bitmap: The routine will create FMaskBmp
  FMaskBmp := TBitmap.Create;
  CreateMaskBitmap(FMaskBmp, Width, Height);
  FMaskBmp.Canvas.Brush.Color := clWhite;                  //Make the mask white
  FMaskBmp.Canvas.FillRect(Rect(0, 0, FMaskBmp.Width, FMaskBmp.Height));

  //Create the icon bitmap
  FIconBmp := TBitmap.Create;
  CreateColorBitmap(FIconBmp, Width, Height);
  FIconBmp.Canvas.Brush.Color := clBlack;                  //Make the icon black
  FIconBmp.Canvas.FillRect(Rect(0, 0, FIconBmp.Width, FIconBmp.Height));

  //NOTE: Having a white mask with a black icon produces a new transparent icon
  UpdateIcon;

  //Fill out the icon info
  FIconCursorInfo.Width := Width;
  FIconCursorInfo.Height := Height;

  //Set the palette size only if color count is lower ot equal to 4BPP: Function will return zero if palette >= 8BPP
  Case BPP of
    ps1Bit, ps4Bit : FIconCursorInfo.ColorCount := GetPaletteSize;
  Else
    FIconCursorInfo.ColorCount := 0;                       //Allways zero when >= 8BPP
  End;


  //Set the X & Y hot spot: For icons this is either 0 or the middle of the icon, depending on your source of information
  FIconCursorInfo.XHotSpot := 0;
  FIconCursorInfo.YHotSpot := 0;
*)
  FIconCursorInfo.BytesInResource := GetIconSize;          //Calculate the icon's size
  FIconCursorInfo.ImageOffset := GetIconOffset;            //This will not be calculated correctly if the icon is not yet added to the list: See TIconCursorList.Add
        
  //Assign event handlers to update HICON
  FMaskBmp.OnChange := BitmapsChange;
  FIconBmp.OnChange := BitmapsChange;
End;



Constructor TIconCursorImage.CreateFromStream(aOwner : TIconCursorList; Stream : TMemoryStream);
Var
  OldPos       : Integer;                                  //The old position of the stream, before reading
  IconResBits  : PByte;                                    //A pointer to where the icon is in the stream
  lpPalette    : PLogPalette;                              //Palette header and color table
  hPal         : hPalette;                                 //Palette Windows handle
  Counter      : Integer;                                  //Loop Counter
  BmpInfo      : TBitmapInfoHeader;                        //Info about the icon
  NumberColors : Integer;                                  //The # of colors
  RGB          : RGBQuad;                                  //A structure for the color table entries


Begin
  Inherited Create;

  //Correct owner ?
  If aOwner.ClassName = 'TIconCursorList' Then             //Can not test it with If aOwner is TIconList: This will allways return true
    FOwner := aOwner                                       //Set the owner
  Else
    Begin
      Stream.Position := OldPos;                           //Restore the stream to the position it was
      Raise EIconException.CreateFmt(ceInvalidOwner, [aOwner.ClassName]);
    End;

  FDestroyIconHandle := FALSE;                             //Set flag to false: Also see CreateIconIndirect & Destructor

  Stream.Read(FIconCursorInfo, SizeOf(FIconCursorInfo));   //First read the info from the stream
  OldPos := Stream.Position;                               //Save the current position: After icon is loaded next icon can be found here

  //Ensure that the API call will not try and read outside the stream: Otherwise we will get an AV
  If NOT ((FIconCursorInfo.ImageOffset + FIconCursorInfo.BytesInResource) <= Stream.Size) Then
  Begin
    Stream.Position := OldPos;                             //Restore the stream to the position it was
    Raise EIconException.CreateFmt(ceCorruptFile, []);
  End;

  IconResBits := Stream.Memory;                            //Set the pointer to the start of the stream
  Inc(IconResBits, FIconCursorInfo.ImageOffset);           //Increment the pointer with the image offset

  //For safety put the API call in a Try...Except
  Try
    //Create an icon from the stream
    FIconHandle := CreateIconFromResourceEx(IconResBits, FIconCursorInfo.BytesInResource, TRUE, $30000, FIconCursorInfo.Width, FIconCursorInfo.Height, LR_DEFAULTCOLOR);
  Except
    Stream.Position := OldPos;                             //Restore the stream to the position it was
    Raise EIconException.CreateFmt(ceCorruptFile, []);
  End;

  //If the API could not create the icon then the file is corrupt
  If FIconHandle = 0 Then
  Begin
    Stream.Position := OldPos;                             //Restore the stream to the position it was
    Raise EIconException.CreateFmt(ceCorruptFile, []);
  End;


  //Create the mask bitmap
  FMaskBmp := TBitmap.Create;
  CreateMaskBitmap(FMaskBmp, FIconCursorInfo.Width, FIconCursorInfo.Height);

  //Draw the icon/cursor mask
  DrawIconEx(FMaskBmp.Canvas.Handle, 0, 0, FIconHandle, FIconCursorInfo.Width, FIconCursorInfo.Height, 0, 0, DI_MASK);


  //Create the icon bitmap
  FIconBmp := TBitmap.Create;
  //Seek gto the offset of the current image to be created and read the header
  Stream.Position := FIconCursorInfo.ImageOffset;
  Stream.Read(BmpInfo, SizeOf(BmpInfo));

  //Check if the constraints of the icon is in range
  If NOT CheckConstraints(BmpInfo.BitCount, BmpInfo.Width, BmpInfo.Height) Then
  Begin
    Stream.Position := OldPos;                             //Restore the stream to the position it was
    Raise EIconException.CreateFmt(ceInvalidSize, [BmpInfo.Width, BmpInfo.Height]);
  End;

  //Set the palette size
  Case BmpInfo.BitCount of
     1 : FPaletteSize := ps1Bit;
     4 : FPaletteSize := ps4Bit;
     8 : FPaletteSize := ps8Bit;
    24 : FPaletteSize := ps24Bit;
    32 : FPaletteSize := ps32Bit;
  Else
    Stream.Position := OldPos;                             //Restore the stream to the position it was
    Raise EIconException.CreateFmt(ceInvalidBPP, [BmpInfo.BitCount]);
  End;

  //If it is not a true color icon then it will have a palette
  If (BmpInfo.BitCount <> 24) AND (BmpInfo.BitCount <> 32) Then
  Begin
    //Get the number of colors in the palette
    NumberColors := GetPaletteSize;

    Try
      //Create a monochrome palette for the mask bitmap
      GetMem(lpPalette, SizeOf(TLogPalette) + (NumberColors * SizeOf(TPaletteEntry)));//Allocate the memory used by the palette

      //Fill out the palette header
      lpPalette^.palVersion := $300;
      lpPalette^.palNumEntries := NumberColors;

      //Turn range checking off if it is on and remember the range checking state
      {$IFOPT R+}
        {$DEFINE CKRANGE}
        {$R-}
      {$EndIF}

      //Fill in the palette structure color table from the stream
      For Counter := 0 to NumberColors - 1 do
      Begin
        Stream.Read(RGB, SizeOf(RGB));

        //There is some confusion: The red and blue bytes seems to be swapped: If swapped here the image colors is swaped, thus it is swapped in GetPaletteColor
        lpPalette^.PalPalEntry[Counter].peRed   := RGB.rgbRed;
        lpPalette^.PalPalEntry[Counter].peGreen := RGB.rgbGreen;
        lpPalette^.PalPalEntry[Counter].peBlue  := RGB.rgbBlue;
      End;

      //Turn range checking back on if it was on when we started
      {$IFDEF CKRANGE}
        {$UNDEF CKRANGE}
        {$R+}
      {$EndIF}

      hPal := CreatePalette(lpPalette^);                     //Create a palette handle
    Finally
      //Free the memory use by the palette structure
      FreeMem(lpPalette, SizeOf(TLogPalette) + (NumberColors * SizeOf(TPaletteEntry)));
    End;
  End;


  //Set the icon bitmap's pixel format based on what is in the bitmap header
  Case BmpInfo.BitCount of
     1 : IconBmp.PixelFormat := pf1Bit;
     4 : IconBmp.PixelFormat := pf4bit;
     8 : IconBmp.PixelFormat := pf8bit;
    16 : IconBmp.PixelFormat := pf16bit;
    24 : IconBmp.PixelFormat := pf24bit;
    32 : IconBmp.PixelFormat := pf32bit;
  Else
    Stream.Position := OldPos;                             //Restore the stream to the position it was
    Raise EIconException.CreateFmt(ceInvalidBPP, [BmpInfo.BitCount]);
  End;

  //The PixelFormat must first be set, before assigning the palette
  If (BmpInfo.BitCount <> 24) AND (BmpInfo.BitCount <> 32) Then
  Begin
    FIconBmp.Palette := CopyPalette(hPal);                 //Copy the palette
    DeleteObject(hPal);                                    //Delete the palette handle after use
  End;

  //Set the width & height of the bitmap
  FIconBmp.Width := FIconCursorInfo.Width;
  FIconBmp.Height := FIconCursorInfo.Height;

  //Use API call to paint the icon/cursor to the canvas
  DrawIconEx(FIconBmp.Canvas.Handle, 0, 0, FIconHandle, FIconCursorInfo.Width, FIconCursorInfo.Height, 0, 0, DI_IMAGE);

  Stream.Position := OldPos;                               //Restore the stream to the position it was

  //Assign event handlers to update HICON
  FMaskBmp.OnChange := BitmapsChange;
  FIconBmp.OnChange := BitmapsChange;
End;



Destructor TIconCursorImage.Destroy;
Begin
  If FDestroyIconHandle Then                               //This condition will only be met if the icon was created using the CreateIconIndirect API
    DestroyIcon(FIconHandle);                              //Free the Windows icon
  If Assigned(FIcon) Then
    FIcon.Free;                                            //Free the Delphi TIcon object
  //Free the icon bitmaps
  If Assigned(FMaskBmp) Then
    FMaskBmp.Free;
  If Assigned(FIconBmp) Then
    FIconBmp.Free;

  Inherited Destroy;
End;




//Get & Set Methods

//Set the palette size
Procedure TIconCursorImage.SetPaletteSize(Const Value: TPaletteSize);
Begin
  FPaletteSize := Value;
  CreateColorBitmap(FIconBmp, FIconBmp.Width, FIconBmp.Height);//Recreate the palette for the color bitmap
End;



//Return the color pointed to by Index
Function TIconCursorImage.GetPaletteColor(Index : Integer ) : TColor;
Var
  PE : TPaletteEntry;

  
Begin
  //Another way to check is to use the Low and High Functions
  If (Index >= 0) AND (Index < GetPaletteSize) Then        //Are we in the boundries?
  Begin
    GetDIBColorTable(FIconBmp.Canvas.Handle, Index, 1, PE);
    //The Red and Blue bytes are swapped so swap them when passing it to the user
    Result := RGB(PE.peBlue, PE.peGreen, PE.peRed);
  End
  Else
    Raise EIconException.CreateFmt(ceInvalidPaletteRange, [Index]);
End;



Procedure TIconCursorImage.SetPaletteColor(Index : Integer; Const Value : TColor);
Var
  PE : TPaletteEntry;


Begin
  If (Index >= 0) AND (Index < GetPaletteSize) Then        //Are we in the boundries of the array?
  Begin
    //The Red and Blue bytes are swapped in the color table so swap them here
    PE.peRed   := GetBValue(Value);
    PE.peGreen := GetGValue(Value);
    PE.peBlue  := GetRValue(Value);
    PE.peFlags := 0;

    SetDIBColorTable(FIconBmp.Canvas.Handle, Index, 1, PE);
  End
  Else
    Raise EIconException.CreateFmt(ceInvalidPaletteRange, [Index]);
End;



Function  TIconCursorImage.GetIconCursorInfo : TIconCursorInfo;
Begin
  Result := FIconCursorInfo;
  //Since the FIconCursorInfo is used for results it is possible that icon is > 4BPP and the ColorCount member is zero: Update it
  Case FPaletteSize of
    ps1Bit          : Result.ColorCount := 1;
    ps4Bit          : Result.ColorCount := 4;
    ps8Bit          : Result.ColorCount := 8;
    ps8BitGrayScale : Result.ColorCount := 8;
    ps24Bit         : Result.ColorCount := 24;
    ps32Bit         : Result.ColorCount := 32;
  End;
End;



Function  TIconCursorImage.GetIsIcon : Boolean;
Begin
  Result := FOwner.IconOrCursor = iocIcon;
End;



Function  TIconCursorImage.GetIsCursor : Boolean;
Begin
  Result := FOwner.IconOrCursor = iocCursor;
End;



Function  TIconCursorImage.GetXHotSpot : Integer;
Begin
  Result := FIconCursorInfo.XHotSpot;
End;



Procedure TIconCursorImage.SetXHotSpot(Value : Integer);
Begin
  If (Value >= 0) AND (Value < FIconCursorInfo.Width) Then
  Begin
    FIconCursorInfo.XHotSpot := Value;
  End;
End;



Function  TIconCursorImage.GetYHotSpot : Integer;
Begin
  Result := FIconCursorInfo.YHotSpot;
End;



Procedure TIconCursorImage.SetYHotSpot(Value : Integer);
Begin
  If (Value >= 0) AND (Value < FIconCursorInfo.Height) Then
  Begin
    FIconCursorInfo.YHotSpot := Value;
  End;
End;



Function  TIconCursorImage.GetXYHotSpot : TPoint;
Begin
  Result.X := GetXHotSpot;
  Result.Y := GetYHotSpot;
End;



Procedure TIconCursorImage.SetXYHotSpot(Value : TPoint);
Begin
  If (Value.X >= 0) AND (Value.X < FIconCursorInfo.Width) AND (Value.Y >= 0) AND (Value.Y < FIconCursorInfo.Height) Then
  Begin
    SetXHotSpot(Value.X);
    SetYHotSpot(Value.Y);
  End;
End;



function TIconCursorImage.GetTransparentPixel(X, Y: Integer): Byte;
Var
  Row : pRGBQuadArray;                                     //Pointer to pixel array data

Begin
  If PaletteSize = ps32Bit Then                            //32 bit icon?
  Begin
    //Range valid?
    If ((X >= 0) AND (X <= fIconBmp.Width)) AND ((Y >= 0) AND (Y <= fIconBmp.Height)) Then
    Begin
      Row := fIconBmp.ScanLine[Y];                         //Get the row
      Result := Row[X].rgbReserved;                        //Return the reserved (Alpha) pixel
    End
    Else
      Raise EIconException.CreateFmt(ceInvalidAlphaBits, [X, Y]);
  End
  Else
    Raise EIconException.Create(ceNoAlphaBits);            //Only 32 bit icons has transparent pixels
End;



procedure TIconCursorImage.SetTransparentPixel(X, Y: Integer; const Value: Byte);
Var
  Row : pRGBQuadArray;                                     //Pointer to pixel array data

Begin
  If PaletteSize = ps32Bit Then                            //32 bit icon?
  Begin
    //Range valid?
    If ((X >= 0) AND (X < fIconBmp.Width)) AND ((Y >= 0) AND (Y < fIconBmp.Height)) Then
    Begin
      Row := fIconBmp.ScanLine[Y];                         //Get the row
      Row[X].rgbReserved := Value;                         //Set the reserved (Alpha) pixel
    End
    Else
      Raise EIconException.CreateFmt(ceInvalidAlphaBits, [X, Y]);
  End
  Else
    Raise EIconException.Create(ceNoAlphaBits);            //Only 32 bit icons has transparent pixels
End;








//Protected Methods

//Check if the BPP and the icon size in range
Function TIconCursorImage.CheckConstraints(Const BPP : TPaletteSize; Const Width, Height : Integer): Boolean;
Begin
  //Check for valid BPP
  Case BPP of
    ps1bit, ps4bit, ps8bit, ps8BitGrayScale, ps24bit, ps32bit : Result := TRUE;
  Else
    Result := FALSE;
  End;

  //Check for valid size
  If NOT (((Width >= cMinIconWidth) AND (Width <= cMaxIconWidth)) AND ((Height >= cMinIconHeight) AND (Height <= cMaxIconHeight))) Then
    Result := FALSE;
End;



//Check if the BPP and the icon size is in range
Function TIconCursorImage.CheckConstraints(Const BPP, Width, Height : Integer): Boolean;
Begin
  //Check for valid BPP
  Case BPP of
    1, 4, 8, 24, 32 : Result := TRUE;
  Else
    Result := FALSE;
  End;

  //Check for valid size
  If NOT (((Width >= cMinIconWidth) AND (Width <= cMaxIconWidth)) AND (((Height DIV 2) >= cMinIconHeight) AND ((Height DIV 2) <= cMaxIconHeight))) Then
    Result := FALSE;
End;



//Create a monochrome bitmap for the icon mask
Procedure TIconCursorImage.CreateMaskBitmap(Const Bitmap : TBitmap; Const Width, Height : Integer);
Var
  lpPalette : PLogPalette;                                 //Palette header and color table
  hPal      : hPalette;                                    //Palette Windows handle
  Counter   : Integer;                                     //Loop Counter
                              
  
Begin
  Try
    //Create a monochrome palette for the mask bitmap
    GetMem(lpPalette, SizeOf(TLogPalette) + (2 * SizeOf(TPaletteEntry)));//Allocate the memory used by the palette
  
    //Fill out the palette header
    lpPalette^.palVersion := $300;
    lpPalette^.palNumEntries := 2;

    //Turn range checking off if it is on and remember the range checking state
    {$IFOPT R+}
      {$DEFINE CKRANGE}
      {$R-}
    {$EndIF}
  
    //Fill in the palette structure color table with black and white: For array acces: Need to use a variable otherwise compiler will get range check error
    Counter := 0;
    lpPalette^.PalPalEntry[Counter].peRed   := $00;
    lpPalette^.PalPalEntry[Counter].peGreen := $00;
    lpPalette^.PalPalEntry[Counter].peBlue  := $00;
    Counter := 1;
    lpPalette^.PalPalEntry[Counter].peRed   := $FF;
    lpPalette^.PalPalEntry[Counter].peGreen := $FF;
    lpPalette^.PalPalEntry[Counter].peBlue  := $FF;
  
    //Turn range checking back on if it was on when we started
    {$IFDEF CKRANGE}
      {$UNDEF CKRANGE}
      {$R+}
    {$EndIF}
  
    hPal := CreatePalette(lpPalette^);                     //Create a palette handle
  Finally
    //Free the memory use by the palette structure
    FreeMem(lpPalette, SizeOf(TLogPalette) + (2 * SizeOf(TPaletteEntry)));
  End;

  //Set the bitmap parameters and assign the palette
  Bitmap.PixelFormat := pf1Bit;                            //Monochrome bitmap
  If Bitmap.Palette <> 0 Then                              //If it has a palette then free it
    Bitmap.ReleasePalette;
  Bitmap.Palette := CopyPalette(hPal);                     //Copy the palette
  DeleteObject(hPal);                                      //Delete the palette handle after use

  //Set the size of the bitmap: Do adjust it if already correct
  If Bitmap.Width <> Width Then
    Bitmap.Width := Width;
  If Bitmap.Height <> Height Then
    Bitmap.Height := Height;
End;




//Creates a color bitmap with the current palette assigned to the HPALETTE of the bitmap
Procedure TIconCursorImage.CreateColorBitmap(Const Bitmap : TBitmap; Const Width, Height : Integer);
Type
  //The resource stream's color format: Dummy bits are inserted between bytes: Most probaply to word allign it
  TResourceColor = Packed Record
    Red    : Byte;
    Dummy1 : Byte;
    Green  : Byte;
    Dummy2 : Byte;
    Blue   : Byte;
    Dummy3 : Byte;
  End;


Var
  Counter             : Integer;                           //Loop Counter
  lpPalette           : PLogPalette;                       //Palette header and color table
  hPal                : hPalette;                          //Palette Windows handle
  PaletteResourceName : String;                            //Name of the resource to load
  ResStream           : TResourceStream;                   //Resource stream to load raw data
  ResColor            : TResourceColor;                    //Structure to read from the resrouce stream

  
Begin
  //If it is a true color icon then do not copy the palette
  If GetPaletteSize <> 0 Then
  Begin
    Try
      //Create a palette for the bitmap
      GetMem(lpPalette, SizeOf(TLogPalette) + (GetPaletteSize * SizeOf(TPaletteEntry)));//Allocate the memory used by the palette
  
      //Fill out the palette header
      lpPalette^.palVersion := $300;
      lpPalette^.palNumEntries := GetPaletteSize;
                    
      //Get the corrcet resource name for the palette
      Case FPaletteSize of
        ps1Bit          : PaletteResourceName := cPaletteRes1Bit;
        ps4Bit          : PaletteResourceName := cPaletteRes4Bit;
        ps8Bit          : PaletteResourceName := cPaletteRes8Bit;
        ps8BitGrayScale : PaletteResourceName := cPaletteRes8BitGray;
      End;                 
      
      //Turn range checking off if it is on and remember the range checking state
      {$IFOPT R+}
        {$DEFINE CKRANGE}
        {$R-}
      {$EndIF}

      //If it is a true color icon then do not load a palette
      If FPaletteSize <> ps24Bit Then
      Begin
        //Load the resource
        Try
          ResStream := TResourceStream.Create(hInstance, PaletteResourceName, RT_RCDATA);

          //Copy the color table into the palette
          For Counter := 0 to (ResStream.Size DIV 6) - 1 do
          Begin
            //Read a color and set it in the palette
            ResStream.Read(ResColor, SizeOf(ResColor));

            //Split the color to its RGB representation and assign it to the palette structure
            lpPalette^.PalPalEntry[Counter].peRed   := ResColor.Red;
            lpPalette^.PalPalEntry[Counter].peGreen := ResColor.Green;
            lpPalette^.PalPalEntry[Counter].peBlue  := ResColor.Blue;
          End;
        Finally
          ResStream.Free;                                      //Free the resource
        End;
      End;            

      //Turn range checking back on if it was on when we started
      {$IFDEF CKRANGE}
        {$UNDEF CKRANGE}
        {$R+}
      {$EndIF}

      hPal := CreatePalette(lpPalette^);                       //Create a palette handle

    Finally
      //Free the memory used by the palette structure
      FreeMem(lpPalette, SizeOf(TLogPalette) + (GetPaletteSize * SizeOf(TPaletteEntry)));
    End;
  End;



  //Set the bitmap parameters and assign the palette
  //Set the BPP of the bitmap:
  Case FPaletteSize of
    ps1Bit          : Bitmap.PixelFormat := pf1Bit;
    ps4Bit          : Bitmap.PixelFormat := pf4Bit;
    ps8Bit          : Bitmap.PixelFormat := pf8Bit;
    ps8BitGrayScale : Bitmap.PixelFormat := pf8Bit;
    ps24Bit         : Bitmap.PixelFormat := pf24Bit;
    ps32Bit         : Bitmap.PixelFormat := pf32Bit;
  End;
  
  //If is not a true color bmp then use the default palette: GetPaletteSize returns 0 for a true color bitmap
  If GetPaletteSize <> 0 Then
  Begin
    If Bitmap.Palette <> 0 Then                            //If the bitmap has a palette then free it 
      Bitmap.ReleasePalette;
    Bitmap.Palette := CopyPalette(hPal);                   //Copy the palette
    DeleteObject(hPal);                                    //Delete the palette handle after use
  End;

  //Set the size of the bitmap: Do adjust it if already correct
  If Bitmap.Width <> Width Then
    Bitmap.Width := Width;
  If Bitmap.Height <> Height Then
    Bitmap.Height := Height;
End;



//Returns the # of colors for the enumerated type: TPaletteSize
Function TIconCursorImage.GetPaletteSize : Integer;
Begin
  //Return the size of the currently selected palette: ie # of colors
  Case FPaletteSize of
    ps1Bit          : Result := Round(IntPower(2, 1));
    ps4Bit          : Result := Round(IntPower(2, 4));
    ps8Bit          : Result := Round(IntPower(2, 8));
    ps8BitGrayScale : Result := Round(IntPower(2, 8));
    ps24Bit,ps32Bit : Result := 0;                         //A true color bitmap does not have a palette
  Else
    Result := 0;                                           //Keeps the compiler happy: Prevent compiler from flagging you about a possible undefined result
  End;
End;


    
//Return a Delphi TIcon object
Function TIconCursorImage.GetIcon : TIcon;
Begin
  If IsIcon Then
  Begin
    If FIcon = Nil Then                                    //Is the icon object created ?
      FIcon := TIcon.Create;
    If FIcon.Handle <> 0 Then                              //Is there an existing icon ?
      FIcon.ReleaseHandle;                                 //Free the GDI object
    FIcon.Handle := FIconHandle;                           //Assign it the icon object
  End
  Else
    //If it not an icon then raise an exception: It will most probaply work for cursors as well
     Raise EIconException.CreateFmt(ceNotAnIcon, []);

  Result := FIcon;                                         //Return the result
End;



//Returns the palette index of the color. -1 if not found or true color
function TIconCursorImage.GetPaletteIndexByColor(const Color: TColor): Integer;
Var
  Counter : Integer;                                       //Loop Counter

begin
  Result := -1;                                            //Default result: -1 (Not found/True color)
  For Counter := 0 to GetPaletteSize - 1 do                //Loop through the palette
    If GetPaletteColor(Counter) = Color Then               //Is there a match?
    Begin                                                  //YES:
      Result := Counter;                                   //Return palette index
      Exit;                                                //Quit loop
    End;
end;



//Returns the # of pixels that it coloured Color
function TIconCursorImage.GetPixelCountOfColor(const Color: TColor): Integer;
Var
  X, Y : Integer;                                          //Loop counters

begin
  Result := 0;                                             //Default result: No pixels has Color as color
  //Loop through the bitmap's pixels:
  For X := 0 to FIconBmp.Width do                          //Width = FIconCursorInfo.Width;
    For Y := 0 to FIconBmp.Height do                       //Height = FIconCursorInfo.Height
      If FIconBmp.Canvas.Pixels[X, Y] = Color Then         //Color matches?
        Inc(Result);                                       //YES: Increment # of pixels
end;



//Creates a maks & image bitmap resulting in a transparent icon
procedure TIconCursorImage.ClearIcon(BPP: TPaletteSize; Width, Height: Integer);
begin
  //Check if all parameters are correct
  If NOT CheckConstraints(BPP, Width, Height) Then
    Raise EIconException.CreateFmt(ceInvalidSize, [Width, Height]);

  FPaletteSize := BPP;                                     //Set palette size before creating the icon bitmap

  //Create the monochrome mask bitmap: The routine will create FMaskBmp
  FMaskBmp := TBitmap.Create;
  CreateMaskBitmap(FMaskBmp, Width, Height);
  FMaskBmp.Canvas.Brush.Color := clWhite;                  //Make the mask white
  FMaskBmp.Canvas.FillRect(Rect(0, 0, FMaskBmp.Width, FMaskBmp.Height));

  //Create the icon bitmap
  FIconBmp := TBitmap.Create;
  CreateColorBitmap(FIconBmp, Width, Height);
  FIconBmp.Canvas.Brush.Color := clBlack;                  //Make the icon black
  FIconBmp.Canvas.FillRect(Rect(0, 0, FIconBmp.Width, FIconBmp.Height));

  //NOTE: Having a white mask with a black icon produces a new transparent icon
  UpdateIcon;

  //Fill out the icon info
  FIconCursorInfo.Width := Width;
  FIconCursorInfo.Height := Height;

  //Set the palette size only if color count is lower ot equal to 4BPP: Function will return zero if palette >= 8BPP
  Case BPP of
    ps1Bit, ps4Bit : FIconCursorInfo.ColorCount := GetPaletteSize;
  Else
    FIconCursorInfo.ColorCount := 0;                       //Allways zero when >= 8BPP
  End;

  //Set the X & Y hot spot: For icons this is either 0 or the middle of the icon, depending on your source of information
  FIconCursorInfo.XHotSpot := 0;
  FIconCursorInfo.YHotSpot := 0;
end;



//Updates the icon recreating the HICON GDI object from the two bitmaps
procedure TIconCursorImage.UpdateIcon;
Var
  IconInfo : Windows.TIconInfo;                            //Windows ICONINFO structure for creating the icon from 2 bitmaps


begin
  If Assigned(FMaskBmp) AND Assigned(FIconBmp) Then
  Begin
    If FMaskBmp.Empty OR FIconBmp.Empty Then
      Exit;
  End
  Else
    Exit;

  If FHoldUpdates Then                                     //User is reponsible to clear flag and call UpdateIcon
    Exit;

  //Create the HICON from the 2 bitmaps: This will also work for cursors: Just clear the flag indicating cursor
  //NOTE: According to the ICONINFO structure the mask bitmap is not used in monochrome icons: It however works on Win 2k in this way.
  If IsIcon Then                                           //Is it an icon
    IconInfo.fIcon := TRUE;                                //Set the flag indicating icon
  If IsCursor Then                                         //Is it a cursor?
  Begin
    IconInfo.fIcon := FALSE;                               //Clear the flag indicating cursor
    //Set the X,Y hot spots
    IconInfo.xHotspot := FIconCursorInfo.XHotSpot;
    IconInfo.yHotspot := FIconCursorInfo.YHotSpot;
  End
  Else
    Begin
      //Set the X,Y hot spots to zero (Depending on what source you use)
      IconInfo.xHotspot := 0;
      IconInfo.yHotspot := 0;
    End;
  IconInfo.hbmMask  := FMaskBmp.Handle;
  IconInfo.hbmColor := FIconBmp.Handle;


  If FDestroyIconHandle Then                               //This condition will only be met if the icon was created using the CreateIconIndirect API
    DestroyIcon(FIconHandle);                              //Free the Windows icon
  FIconHandle := CreateIconIndirect(IconInfo);
  If FIconHandle <> 0 Then                                 //Did the function succeed ?
    FDestroyIconHandle := TRUE;                            //Set a flag used by the destructor
end;



//Paints the icon to a canvas: The Brush color of the canvas passed in aCanvas is used as transparent color
Procedure TIconCursorImage.Paint(Const aCanvas : TCanvas; Const Left, Top : Integer);
Begin
  Windows.DrawIconEx(aCanvas.Handle, Left, Top, FIconHandle, FIconCursorInfo.Width, FIconCursorInfo.Height, 0, 0, DI_NORMAL);
End;



//Save an icon to a file: This method can only be used to save a single icon to a file
Procedure TIconCursorImage.SaveToFile(Const FileName: String);
Var
  TmpStream      : TMemoryStream;                          //A temp stream to save the icon to before dumping it in a file
  IconFileHeader : TIconCursorHeader;                      //The icon/cursor file header
  IconHeader     : TIconCursorInfo;                        //The icon/cursor header


Begin
  Try
    TmpStream := TMemoryStream.Create;                     //Create a temp memory stream

    //Fill out the icon header
    IconFileHeader.Reserved := 0;                          //Must be zero
    If IsIcon Then
      IconFileHeader.ResType := cIcon;                     //It is an icon
    If IsCursor Then
      IconFileHeader.ResType := cCursor;                   //It is a cursor
    IconFileHeader.Count := 1;                             //Only this icon is in the icon file

    //Save the icon file header: 6 Bytes
    TmpStream.Write(IconFileHeader, SizeOf(IconFileHeader));//Save the main icon header in the stream

    //Save the icon header: Do not use SaveHeaderToStream since it will involve too much extra code (If there are more than one icon in the icon list then it will envolve seeking back and chaning the offsets.)
    IconHeader := FIconCursorInfo;                         //Get value
    IconHeader.BytesInResource := GetIconSize;             //Ensure that it is correct
    IconHeader.ImageOffset := SizeOf(IconFileHeader) + SizeOf(IconHeader);//The image offset for only 1 icon in a file
    TmpStream.Write(IconHeader, SizeOf(IconHeader));       //Save icon info to file

    //Save the image bits
    SaveImagesToStream(TmpStream);

    TmpStream.SaveToFile(FileName);                        //Save the stream to a file
  Finally
    TmpStream.Free;                                        //Free the stream
  End;
End;



//Save an icon to a stream
Procedure TIconCursorImage.SaveHeaderToStream(Const Stream : TMemoryStream);
Begin
  //Make sure this info is current
  FIconCursorInfo.ImageOffset := GetIconOffset;
  FIconCursorInfo.BytesInResource := GetIconSize;

  Stream.Write(FIconCursorInfo, SizeOf(FIconCursorInfo));  //Save the header to the stream
End;



//Save the icon images to a stream: Saving starts at the current position of the stream
Procedure TIconCursorImage.SaveImagesToStream(Const Stream : TMemoryStream);
Var
  MaskStream       : TMemoryStream;                        //A temp stream to save the mask bitmap to
  IconStream       : TMemoryStream;                        //A temp stream to save the icon bitmap to
  IconHeaderSize   : Cardinal;                             //The header size of the icon bitmap
  IconImageSize    : Cardinal;                             //The size of the image of the icon bitmap
  MaskHeaderSize   : Cardinal;                             //The header size of the mask bitmap
  MaskImageSize    : Cardinal;                             //The size of the image of the mask bitmap
  BitmapInfoHeader : TBitmapInfoHeader;                    //The bitmap info header
  Counter          : Integer;                              //General loop counter
  aColor           : TColor;                               //The color to be saved
  RGBQuad          : TRGBQuad;                             //The RGB Quad structure for the palette



Begin
  //Get the size of the mask bitmap image: (Header is not used)
  GetDIBSizes(FMaskBmp.Handle, MaskHeaderSize, MaskImageSize);
  //Get the size of the icon image bitmap and the header
  GetDIBSizes(FIconBmp.Handle, IconHeaderSize, IconImageSize);

  Try
    MaskStream := TMemoryStream.Create;                    //Create a temp memory stream
    IconStream := TMemoryStream.Create;                    //Create a temp memory stream
    FMaskBmp.SaveToStream(MaskStream);                     //Save the mask bitmap to the stream
    FIconBmp.SaveToStream(IconStream);                     //Save the icon bitmap to the stream


    //Fill out the bitmap info header
    BitmapInfoHeader.Size := SizeOf(BitmapInfoHeader);
    BitmapInfoHeader.Width  := FIconBmp.Width;
    BitmapInfoHeader.Height := FMaskBmp.Height + FIconBmp.Height;//The height must be the combined height of the 2 bitmaps
    BitmapInfoHeader.Planes := 1;                          //This field is set to 1 but it is unknown and possible that some icons will have it set to 0
    //Set the bit count of the bitmap info header
    Case FPaletteSize of
      ps1Bit          : BitmapInfoHeader.BitCount := 1;
      ps4Bit          : BitmapInfoHeader.BitCount := 4;
      ps8Bit          : BitmapInfoHeader.BitCount := 8;
      ps8BitGrayScale : BitmapInfoHeader.BitCount := 8;
      ps24Bit         : BitmapInfoHeader.BitCount := 24;
      ps32Bit         : BitmapInfoHeader.BitCount := 32;
    End;
    BitmapInfoHeader.Compression := 0;
    BitmapInfoHeader.SizeImage := IconImageSize + MaskImageSize;//The size of the two bitmaps' bits
    BitmapInfoHeader.XPelsPerMeter := 0;
    BitmapInfoHeader.YPelsPerMeter := 0;
    BitmapInfoHeader.ClrUsed := GetPaletteSize;            //It is uncertain if this field is used by all software that loads icons...
    BitmapInfoHeader.ClrImportant := 0;

    Stream.Write(BitmapInfoHeader, SizeOf(BitmapInfoHeader));//Write the bitmap info header to the stream


    //Save the palette
    For Counter := 0 to GetPaletteSize - 1 do
    Begin
      aColor := PaletteColor[Counter];                     //Get the color from the array

      //Split the color in to its respective RGB values
      RGBQuad.rgbRed   := GetRValue(aColor);
      RGBQuad.rgbGreen := GetGValue(aColor);
      RGBQuad.rgbBlue  := GetBValue(aColor);

      Stream.Write(RGBQuad, SizeOf(RGBQuad));              //Write the value to the stream
    End;

    //Copy the icon bits
    IconStream.Position := SizeOf(TBitmapFileHeader) + IconHeaderSize;//Both needs an extra offset of 14 bytes: I assume it is the bitmap file header that must be added
    Stream.CopyFrom(IconStream, IconStream.Size - IconStream.Position);

    //Copy the mask bits
    MaskStream.Position := SizeOf(TBitmapFileHeader) + MaskHeaderSize;//Both needs an extra offset of 14 bytes: I assume it is the bitmap file header that must be added
    Stream.CopyFrom(MaskStream, MaskStream.Size - MaskStream.Position);

  Finally
    IconStream.Free;                                       //Free the temp stream
    MaskStream.Free;                                       //Free the temp stream
  End;
End;



//Calculate the size of the icon
Function TIconCursorImage.GetIconSize : Cardinal;
Var
  IconHeaderSize : Cardinal;                               //The header size of the icon bitmap
  IconImageSize  : Cardinal;                               //The size of the image of the icon bitmap
  MaskHeaderSize : Cardinal;                               //The header size of the mask bitmap: Not used but needed
  MaskImageSize  : Cardinal;                               //The size of the image of the mask bitmap

  
Begin
  //Get the size of the mask bitmap image: (Header is not used)
  GetDIBSizes(FMaskBmp.Handle, MaskHeaderSize, MaskImageSize);
  //Get the size of the icon image bitmap and the header
  GetDIBSizes(FIconBmp.Handle, IconHeaderSize, IconImageSize);

  //The size of the icon header structure + mask bitmap bits + palette & icon bitmap header + icon bitmap bits 
  Result := MaskImageSize + IconHeaderSize + IconImageSize;
End;



//Calculate the offst of this icon of TIconCursorImage in the TIconList object
Function TIconCursorImage.GetIconOffset : Cardinal;
Var
  Counter : Integer;                                       //Loop counter


Begin
  Result := SizeOf(TIconCursorHeader) + SizeOf(TIconCursorInfo) * FOwner.Count;//All icon files contains this header

  //Loop through from the second icon to yourself in the icon list and add the sizes
  For Counter := 1 to FOwner.IndexOf(Self) do
    Result := Result + FOwner.Items[Counter - 1].GetIconSize //It is safer to call GetIconSize than to use FIconInfo.BytesInResource
End;



//Event handler to recreate the HICON when bitmaps change
procedure TIconCursorImage.BitmapsChange(Sender: TObject);
begin
  UpdateIcon;                                              //Recreate the icon handle
end;



//Allow TIcon to copy it's properties to TIconCursorImage
procedure TIconCursorImage.Assign(Source : TPersistent);
Var
  Counter : Integer;                                       //Loop Counter


begin
  If (Source is TIconCursorImage) Then                     //Is it a TIconCursorImage object ?
  Begin
    FIconCursorInfo := TIconCursorImage(Source).FIconCursorInfo;//Set the icon/cursor info
    If FDestroyIconHandle Then                             //This condition will only be met if the icon was created using the CreateIconIndirect API
    Begin
      DestroyIcon(FIconHandle);                            //Free the Windows icon
      FDestroyIconHandle := FALSE;                         //It was not created using: CreateIconIndirect API call
    End;
    FIconHandle := CopyIcon(TIconCursorImage(Source).FIconHandle);//Copy the Windows GDI object


    //FIconBmp.Assign(TIconCursorImage(Source).FIconBmp);  //Copy the icon bitmap
    //Can not use TBitmap.Assign due to bugs in Delphi 4: The following lines of code does the same
    FIconBmp.Width := TIconCursorImage(Source).FIconBmp.Width;
    FIconBmp.Height := TIconCursorImage(Source).FIconBmp.Height;
    FIconBmp.PixelFormat := TIconCursorImage(Source).FIconBmp.PixelFormat;
    FIconBmp.Canvas.Draw(0, 0, TIconCursorImage(Source).FIconBmp);
    FIconBmp.FreeImage;                                    //This may not be neccessary
    If TIconCursorImage(Source).FIconBmp.Palette <> 0 Then //Is there a palette?
    Begin
      If FIconBmp.Palette <> 0 Then                        //Must the current palette be freed?
        FIconBmp.ReleasePalette;
      FIconBmp.Palette := CopyPalette(TIconCursorImage(Source).FIconBmp.Palette);
    End;


    //FMaskBmp.Assign(TIconCursorImage(Source).FMaskBmp);  //Copy the mask bitmap
    //Can not use TBitmap.Assign due to bugs in Delphi 4: The following lines of code does the same
    FMaskBmp.Width := TIconCursorImage(Source).FMaskBmp.Width;
    FMaskBmp.Height := TIconCursorImage(Source).FMaskBmp.Height;
    FMaskBmp.PixelFormat := TIconCursorImage(Source).FMaskBmp.PixelFormat;
    FMaskBmp.Canvas.Draw(0, 0, TIconCursorImage(Source).FMaskBmp);
    FMaskBmp.FreeImage;                                    //This may not be neccessary
    If TIconCursorImage(Source).FMaskBmp.Palette <> 0 Then //Is there a palette?
    Begin
      If FMaskBmp.Palette <> 0 Then                        //Must the current palette be freed?
        FMaskBmp.ReleasePalette;
      FMaskBmp.Palette := CopyPalette(TIconCursorImage(Source).FMaskBmp.Palette);
    End;

    //Palette
    FPaletteSize := TIconCursorImage(Source).FPaletteSize; //Set the palette size

    //Update all the image offset values
    For Counter := 0 to FOwner.FList.Count - 1 do
      FOwner.GetIconImage(Counter).FIconCursorInfo.ImageOffset := GetIconOffset;
  End
  Else
    If (Source is Graphics.TIcon) Then                     //Is it a TIcon object ?
    Begin
      If FDestroyIconHandle Then                           //Must it be freed?
      Begin
        FDestroyIconHandle := False;                       //Clear the flag
        DestroyIcon(FIconHandle);                          //Free the Windows icon
      End;

      //Put all this code in Try..Excep block to ensure valid icon remains even when assignment fails
      Try
        //For safety put the API call in a Try...Except
        Try
          //Copy the icon:
          fIconHandle := Windows.CopyIcon(Graphics.TIcon(Source).Handle);
        Except
          Raise EIconException.Create(ceCopyFailed);
        End;

        //If the API could not copy the icon raise exception
        If FIconHandle = 0 Then
          Raise EIconException.Create(ceCopyFailed);

        fIconCursorInfo.Width := Graphics.TIcon(Source).Width;
        fIconCursorInfo.Height := Graphics.TIcon(Source).Height;
        fIconCursorInfo.ColorCount := 0;
        fIconCursorInfo.Reserved := 0;
        //Info lost since TIcon does not supply hot spots!
        fIconCursorInfo.XHotSpot := 0;
        fIconCursorInfo.YHotSpot := 0;


        //Create the mask bitmap
        FMaskBmp := TBitmap.Create;
        CreateMaskBitmap(FMaskBmp, FIconCursorInfo.Width, FIconCursorInfo.Height);

        //Draw the icon/cursor mask
        DrawIconEx(FMaskBmp.Canvas.Handle, 0, 0, FIconHandle, FIconCursorInfo.Width, FIconCursorInfo.Height, 0, 0, DI_MASK);

        //Create the icon bitmap
        FIconBmp := TBitmap.Create;

        //The PixelFormat must first be set, before assigning the palette
        If (Graphics.TIcon(Source).Palette <> 0 ) Then
          FIconBmp.Palette := CopyPalette(Graphics.TIcon(Source).Palette);//Copy the palette

        //Set the width & height of the bitmap
        FIconBmp.Width := FIconCursorInfo.Width;
        FIconBmp.Height := FIconCursorInfo.Height;

        //Use API call to paint the icon/cursor to the canvas
        DrawIconEx(FIconBmp.Canvas.Handle, 0, 0, FIconHandle, FIconCursorInfo.Width, FIconCursorInfo.Height, 0, 0, DI_IMAGE);

        fIconCursorInfo.BytesInResource := GetIconSize;    //Calculate the icon's size
        fIconCursorInfo.ImageOffset := GetIconOffset;      //This will not be calculated correctly if the icon is not yet added to the list: See TIconCursorList.Add
      Except
        //Always provide a default icon:
        ClearIcon(Self.PaletteSize, Self.IconInfo.Width, Self.IconInfo.Height);
        Raise;
      End;
    End
    Else
      Inherited Assign(Source);                            //Give error message
end;



//Allow TIconCursorImage to copy to TIcon
procedure TIconCursorImage.AssignTo(Dest: TPersistent);
begin
  If (Dest is TIcon) Then
  Begin
    Graphics.TIcon(Dest).ReleaseHandle;                    //Free the GDI object
    Graphics.TIcon(Dest).Handle := Self.Handle;            //Set the new handle
  End
  Else
    Inherited AssignTo(Dest);
end;













//Paints the icon replacing the transparent parts with TransparentColor and the inverse parts with InverseColor. The result is returned in ResultBmp.
//All 3 bitmaps must be the same size. ResultBmp must also be created prior to calling the method.
//This routine does not really check for exceptions, since it is very unlikely: If it occurs there is no way of telling...
Procedure PaintIcon(Const TransparentColor, InverseColor : TColor; IconBmp, MaskBmp, ResultBmp : TBitmap);


  //This method will replace the transparent parts of the icon bitmaps with the TransparentColor color. The other parts of the image are unchanged. It will
  //leave the parts to be inverted in white. All 3 bitmaps must be the same size. ResultBmp must also be created prior to calling the method.
  Procedure PaintTransparentBmp(Const TransparentColor : TColor; Const IconBmp, MaskBmp, ResultBmp : TBitmap);
  Var
    OldBrushColor : TColor;                                //The color of the brush when the function enters


  Begin
    Try
      OldBrushColor := ResultBmp.Canvas.Brush.Color;       //Save the brush color
      ResultBmp.Canvas.Brush.Color := TransparentColor;    //Set it to the transparent color
      ResultBmp.Canvas.FillRect(Rect(0, 0, ResultBmp.Width, ResultBmp.Height));//Fill the entire result bitmap with the transparent color

      //AND the pixels of the mask bitmap and the result bitmap: Result bitmap is changed
      BitBlt(ResultBmp.Canvas.Handle, 0, 0, ResultBmp.Width, ResultBmp.Height, MaskBmp.Canvas.Handle, 0, 0, SRCAND);
      //OR the pixels of the result bitmap and the icon bitmap: Result bitmap is changed again
      BitBlt(ResultBmp.Canvas.Handle, 0, 0, ResultBmp.Width, ResultBmp.Height, IconBmp.Canvas.Handle, 0, 0, SRCPAINT);

    Finally
      ResultBmp.Canvas.Brush.Color := OldBrushColor;       //Restore the brush color
    End;
  End;



  //This method will XOR the two bitmaps. The inverse part of the icon is painted in black, the rest of the icon are just copied, except for the transparent
  //part, which is painted in white. All 3 bitmaps must be the same size. ResultBmp must also be created prior to calling the method.
  Procedure PaintInverseIconBmp(Const IconBmp, MaskBmp, ResultBmp : TBitmap);
  Var
    TmpBmp : TBitmap;                                      //The inverted icon bitmap

  Begin
    //This is what actually happens: Both TmpBmp and ResultBmp are used a temp bitmaps, like C & D
    //C = Mask AND (NOT Icon)
    //D = Icon AND (NOT Mask)
    //R = C OR D

    Try
      //Create the temp bitmap and set it's size
      TmpBmp := TBitmap.Create;
      TmpBmp.Width := IconBmp.Width;
      TmpBmp.Height := IconBmp.Height;
      TmpBmp.PixelFormat := pf24Bit;                       //Allways use true color and not device, since device may be lower

      //Copy the inverted icon bitmap to TmpBmp
      BitBlt(TmpBmp.Canvas.Handle, 0, 0, IconBmp.Width, IconBmp.Height, IconBmp.Canvas.Handle, 0, 0, SRCINVERT);
      //Copy the inverted mask bitmap to ResultBmp
      BitBlt(ResultBmp.Canvas.Handle, 0, 0, MaskBmp.Width, MaskBmp.Height, MaskBmp.Canvas.Handle, 0, 0, SRCINVERT);

      //AND mask bitmap with TmpBmp: Result is stored in TmpBmp
      BitBlt(TmpBmp.Canvas.Handle, 0, 0,  TmpBmp.Width, TmpBmp.Height, MaskBmp.Canvas.Handle, 0, 0, SRCAND);
      //AND ResultBmp with IconBmp: Result is store in ResultBmp
      BitBlt(ResultBmp.Canvas.Handle, 0, 0,  ResultBmp.Width, ResultBmp.Height, IconBmp.Canvas.Handle, 0, 0, SRCAND);

      //OR ResultBmp and TmpBmp: Result is stored in result
      BitBlt(ResultBmp.Canvas.Handle, 0, 0, TmpBmp.Width, TmpBmp.Height, TmpBmp.Canvas.Handle, 0, 0, SRCPAINT);

    Finally
      //Free the temp bitmap
      TmpBmp.Free;
    End;
  End;



  //Paint a black image with the inverse color painted in InverseColor. This yiels a bitmap which is black except at the InverseColor pixels it is painted in
  //InverseColor. All 3 bitmaps must be the same size. ResultBmp must also be created prior to calling the method.
  Procedure PaintInverseMaskBmp(Const InverseColor : TColor; Const IconBmp, MaskBmp, ResultBmp : TBitmap);
  Var
    OldBrushColor : TColor;                                //The color of the brush when the function enters

  Begin
    //What actually happens is this
    //InverseColorBmp = Bitmap filled with the inverse color
    //TmpBmp = MaskBmp AND InverseColorBmp
    //Result = ColorBmp OR TmpBmp

    Try
      OldBrushColor := ResultBmp.Canvas.Brush.Color;       //Save the brush color
      ResultBmp.Canvas.Brush.Color := InverseColor;        //Set the brush color to the inverse color
      ResultBmp.Canvas.FillRect(Rect(0, 0, ResultBmp.Width, ResultBmp.Height));//Fill the bitmap with the inverse color

      //AND the mask bitmap and the filled inverse color bitmap: Result in ResultBmp
      BitBlt(ResultBmp.Canvas.Handle, 0, 0, ResultBmp.Width, ResultBmp.Height, MaskBmp.Canvas.Handle, 0, 0, SRCAND);
      //AND the icon bitmap and the result bitmap: Result in result bitmap
      BitBlt(ResultBmp.Canvas.Handle, 0, 0, ResultBmp.Width, ResultBmp.Height, IconBmp.Canvas.Handle, 0, 0, SRCAND);

    Finally
      ResultBmp.Canvas.Brush.Color := OldBrushColor;       //Restore the brush color
    End;
  End;



Var
  TmpBmp : TBitmap;                                        //A temp bitmap

Begin
  //What actually happens is this:
  //Result = PaintTransparentBmp AND (PaintInverseMask OR PaintInverseIconBmp)

  Try
    //Create a temp bitmap and set it's size
    TmpBmp := TBitmap.Create;
    TmpBmp.Width := IconBmp.Width;
    TmpBmp.Height := IconBmp.Height;
    TmpBmp.PixelFormat := pf24Bit;                         //Allways use true color and not device, since device may be lower

    //Paint a black image with inverse color painted as InverseColor: Result in ResultBmp
    PaintInverseMaskBmp(InverseColor, IconBmp, MaskBmp, ResultBmp);
    //XOR the icon and mask bitmap resulting in black and white reversed, but nothing else changed: Result in TmpBmp
    PaintInverseIconBmp(IconBmp, MaskBmp, TmpBmp);

    //OR the above results: New result in TmpBmp
    BitBlt(TmpBmp.Canvas.Handle, 0, 0, TmpBmp.Width, TmpBmp.Height, ResultBmp.Canvas.Handle, 0, 0, SRCPAINT);
    //Paint the transparent parts of the icon in TransparentColor: Result in ResultBmp
    PaintTransparentBmp(TransparentColor, IconBmp, MaskBmp, ResultBmp);
    //AND the previous result together producing the final icon
    BitBlt(ResultBmp.Canvas.Handle, 0, 0, ResultBmp.Width, ResultBmp.Height, TmpBmp.Canvas.Handle, 0, 0, SRCAND);

  Finally
    //Free the temp bitmap
    TmpBmp.Free;
  End;
End;





Procedure AlphaPaintIcon(Const TransparentColor, InverseColor : TColor; IconBmp, MaskBmp, ResultBmp : TBitmap);
Begin
  If IconBmp.PixelFormat <> pf32Bit Then                   //32 bit bitmap (Contains Alpha Channel)?
    Exit;                                                  //NO: Quit

  //Draw the icon just as it is normally done:
  PaintIcon(TransparentColor, InverseColor, IconBmp, MaskBmp, ResultBmp);

  //Now blend the pixels with the alpha channel:
//  use scanline access since it is fast
//  convert mask bmp to 32 bit bmp to operate quickly on it
End;





End.



{

Icon Color | Mask Color | Picture Color
=======================================
Black        Black        Black
White        Black        White
Black        White        Transparent
White        White        Inverse
Any          Black        Any



TIconImage
==========
1. It may seem strange that TIconImage has a SaveToFile method but not a LoadFromFile method. This is due to the fact that a single icon can be saved
   to an icon file. When loading an icon from a file there is no guarantee that there will be only one icon in the file. For this reason the load must
   be attempted through the TIconCursorList.
2. Procedure TIconImage.UpdateIcon; The method may possibly not work on other platforms. It works on Win XP. The method uses tha API call CreateIconIndirect to
                                    create the icon. API documentation specifies in the ICONINFO structure documentation that the mask bitmap is not used to
                                    create a monochrome icon. This however works under Win XP. Not tested on other platforms
3. The color table seems to be a problem: All icons load and save correctly. The Red and Blue bytes of the color table seems to be swapped. Upon creating the
   color table the bytes are not swapped and the palette is created with the swapped bytes. When the used reads or writes the palette the bytes is swapped to
   ensure consistency. I do not know if al palettes loads corrcetly or not, but it seems so. 


Limitations
===========
The TIconImage object can not be used as a stand alone object: eg. If you create an instance of TIconImage, not using the list but add it to the list,
and you free the list, the TIconImage instance will also be freed.
(It does not realy make any sense to use the TIconImage on its own, it must be part of the list.)




Bit changes in files
====================

The following combinations was tested and also some combinations thereof:

1BPP:   8 x  8 *
       16 x 16 *
       32 x 32
       48 x 48 *
       64 x 64

4BPP:   8 x  8 *
       16 x 16 *
       32 x 32
       48 x 48 *
       64 x 64

8BPP:   8 x  8 *
       16 x 16 *
       32 x 32
       48 x 48
       64 x 64

24BPP:  8 x  8 *
       16 x 16 *
       32 x 32
       48 x 48 *
       64 x 64

* Some of the mask bits in the original file are changed from 00h to FFh etc: The icons however display the same and seems unchanged. I do not know why this
  happens. A similar error was reported by other programmers in saving a bitmap.



  
The icon file format
====================

The following will try and explain the icon file format using close to Pascal syntax.
Please note that the structures refers to one another and are in the correct order.
For the correct type please refer to IconCursorTypes.Pas. This types are modified to
try and explain how the icon file is constructed.


  //The icon file header
  TIconHeader = Record                 //6 Bytes
    Reserved : Word;                   //Must be 0
    ResType  : Word;                   //Resource type: 1 for icons; 2 for cursors
    Count    : Word;                   //# of images in file
    IconCursorInfo : Array[0..Count] of TIconCursorInfo;//Array of Count TIconCursorInfo structures (One for each icon)
  End;

  
  //The header for each icon in an icon file: The XHotSpot & YHotSpot is in some windows documentation replaced by Planes & BPP: This is however wrong
  TIconCursorInfo = Packed Record      //16 Bytes
    Width           : Byte;            //Width in pixels of the image
    Height          : Byte;            //Height in pixels of the image
    ColorCount      : Byte;            //# of colors in the image (0 if >= 8bpp)
    Reserved        : Byte;            //Must be 0
    XHotSpot        : Word;            //X &
    YHotSpot        : Word;            //Y hotspots (Both is ignored for icons and should be zero)
    BytesInResource : Cardinal;        //# of bytes in resource
    ImageOffset     : Cardinal;        //Image start offset from begin of file
  End;


  //An array of all the icons (The sizes of the elements in the array will not be the same.)
  Array[0..Count] of IconImage = Record
                       BmpHeader  : TBitmapInfoHeader;                 //The bitmap info header: Same as Windows.tagBITMAPINFOHEADER or Windows.TBitmapInfoHeader
                                                                       //(Used from IconCursorTypes.Pas) 40 Bytes
                       ColorTable : Array[1..2 ^ BitCount] of RGBQuad; //The color table from which a palette must be generated
                                                                       //The RGBQuad structure is a structure with an entry for Red, Green, Blue and a reserved byte
                                                                       //this structure is defined in Windows.Pas
                                                                       //ColorTable is an array of RGBQuad with one entry for each color the palette must have
                                                                       //2 ^ BitCount = 2 to the power of the BPP of the bitmap/icon
                                                                       //The only sizes accepted for BitCount are: 1 (2 Colors), 4 (16 Colors), 8 (256 Colors), 24 (True Color), 32 (True Color with Alpha Trasparent Byte)
                                                                       //NOTE: The ColorTable is omitted if it is a true color bitmap/icon
                       XORMask    : Array of Byte;                     //The color part of the icon: The bits the bitmap/icon consists of
                       ANDMask    : Array of Byte;                     //The mask part of the icon: The bits the bitmap/icon consists of
                     End;

                     
NOTES: 1. The BmpHeader.Height member indicates the XORMask + the ANDMask height.
       2. The ANDMask is allways a monochrome bitmap and has no BmpHeader.
       3. For true color icons there are no color table.
       4. The reserved byte is used to indicate transparent pixels in the TRGBQuad structure.
       5. The TIconCursorImage class was not derived from TGraphic since a list of TIconCursorImage is used and a lot of methods would not make sense if implimented.

}


