// Copyright: This source code is copyrighted (c) 2006 by Johan Erasmus. All rights reserved. This source code shall remain the exclusive
//            property of the author(s).
//
// Distribution: You are granted a non-exlusive, royalty-free right to produce and distribute compiled binary files that are built with any
//               of this source code unless specifically stated otherwise.
//               Without the express written consent of the author, you may not:
//               - Distribute modified versions of any source code by itself. You must include the original source code as you found it at
//                 the author(s)' web site.
//               - Sell or lease any portion of this source code. You are free to sell any of your own original code that works with or
//                 enhances this source code.
//               - Distribute this source code for profit.
//
// Disclaimer: This source code is provided 'as is' with no warranty of any kind.
//             The author disclaims all warranties, expressed or implied, including, without limitation, the warranties of merchantability
//             and of fitness for any purpose. Johan Erasmus assumes no liability for damages, direct or consequential, which may result
//             from the use of this source code. You also agree to not hold Johan Erasmus liable for any damages, direct or consequential
//             which may result from the use of this source code. By using this source code, you are agreeing to all of these conditions.
//
// Support: You are free to contact me, but I can not gaurantee any support whatsoever. I will however try to answer all question that are
//          directed to me.
//
// Contact Information: jerasmus@flashmail.com
//
//
// Date convention: dd/mm/yyyy
// Date Started: 21/05/2002
// Date Completed: 03/09/2002
// Date Modified: 02/01/2003 - Fixed TIconCursorInfo structure
//                23/03/2004 - Added TIconCursorSortCriteria & TIconCursorSortOrder
//                13/10/2006 - Added definitions for Alpha Transparency icons
//
// Version: 3.00
// Compiler: Borland Delphi 5 Profesional (UP 1)
// Delphi Support: 4, 5, 6 (Tested only on 5)
// OS Platforms Supported: Win 9x, Win ME, Win NT 4, Win 2k, Win XP (ps32Bit only on XP)
// OS Platforms Tested: Win XP (SP 2)
//
// Description: Types declared for use with Icon.PAS
// Developed by: Johan Erasmus
//


Unit IconCursorTypes;

Interface

Uses Windows, Math;



Type
  //The header of an icon and cursor file
  TIconCursorHeader = Packed Record
    Reserved : Word;                   //Must be 0
    ResType  : Word;                   //Resource type: 1 for icons
    Count    : Word;                   //# of images in file
  End;


  //The header for each icon in an icon file: The XHotSpot & YHotSpot is in some windows documentation replaced by Planes & BPP: This is however wrong
  TIconCursorInfo = Packed Record
    Width           : Byte;            //Width in pixels of the image
    Height          : Byte;            //Height in pixels of the image
    ColorCount      : Byte;            //# of colors in the image (0 if >= 8bpp)
    Reserved        : Byte;            //Must be 0
    XHotSpot        : Word;            //X &
    YHotSpot        : Word;            //Y hotspots (Both is ignored for icons and should be zero)
    BytesInResource : Cardinal;        //# of bytes in resource
    ImageOffset     : Cardinal;        //Image start offset from begin of file
  End;


  //This is the same as the tagBITMAPINFOHEADER in Windows.Pas
  //The header for each icon: Describes the bitmap
  TBitmapInfoHeader = Packed Record
    Size          : DWORD;             //This is the size of this structure: Add this size to get to the color table
    Width         : Longint;           //The width of the bitmap in pixels
    Height        : Longint;           //The height of the bitmap in pixels
    Planes        : Word;              //Must be 1
    BitCount      : Word;              //The BPP or color resolution: 1, 4, 8, 24, 32
    Compression   : DWORD;             //Must be 0
    SizeImage     : DWORD;             //The size of the bitmap
    XPelsPerMeter : Longint;           //Must be 0
    YPelsPerMeter : Longint;           //Must be 0
    ClrUsed       : DWORD;             //When less colors are used than the amount of colors defined in the color table the value must be set here
    ClrImportant  : DWORD;             //Specifies that the first n colors of the color table are important
  End;


  //This is the palette sizes supported:
  TPaletteSize = (ps1Bit, ps4Bit, ps8Bit, ps8BitGrayScale, ps24Bit, ps32Bit);

  //Flag to indicate if the file is an icon or cursor:
  TIconOrCursor = (iocIcon, iocCursor);

  //Sort order:
  TIconCursorSortCriteria = (icscNone, icscColorDepth, icscSize);

  //Ascending or descending sort order:
  TIconCursorSortOrder = (icsoAscending, icsoDescending);

  //Structures to access pixel data for Alpha Trasparent bitmaps:
  pRGBQuadArray = ^TRGBQuadArray;                          //Pointer to the bits
  TRGBQuadArray = ARRAY[0..255] OF TRGBQuad;               //Array size need not be larger than the largest icon: Max icon size limited by Byte size
                                                           //                                                     since TIconCursorInfo.Widht & TIconCursorInfo.Height
                                                           //                                                     are bytes only


Implementation



End.


